

YssDrop = CreateFrame("Frame")
local YssDrop = YssDrop
YssDrop:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
YssDrop:RegisterEvent("ADDON_LOADED")

local QTC = LibStub('LibQTipClick-1.0')
local broker = LibStub("LibDataBroker-1.1")
local icon = LibStub("LibDBIcon-1.0")

local frameStrata = {"BACKGROUND", "LOW", "MEDIUM", "HIGH", "DIALOG", "FULLSCREEN", "FULLSCREEN_DIALOG", "TOOLTIP"}

function YssDrop:Print(...) ChatFrame1:AddMessage(string.join(" ", "|cFF33FF99Broker: YssDrop|r:", ...)) end

-- QTC hide function
local tooltip
function YssDrop:HideTooltip()
	self:SetScript('OnUpdate', nil)
	tooltip:SetFrameStrata("TOOLTIP")
	tooltip:SetScript( "OnLeave", nil )
	tooltip:Clear()
	tooltip:Hide()
	tooltip = QTC:Release( tooltip )
end

-- Broker Tooltip handling
local GameTooltip = GameTooltip

local function GetAnchors(cell)
	local x, y = cell:GetCenter()
	local vertical
	if y < GetScreenHeight() / 2 then
		vertical = "BOTTOM"
	else
		vertical = "TOP"
	end
	if x < GetScreenWidth() / 2 then
		return vertical.."LEFT", vertical.."RIGHT"
	else
		
		return vertical.."RIGHT", vertical.."LEFT"
	end
end

local function GT_OnLeave(self)
	self:SetScript("OnLeave", self.YssDrop_OnLeave)
	self:Hide()
	GameTooltip:EnableMouse(false)
end

local function PrepareTooltip(frame, cell)
	if frame == GameTooltip then
		frame.YssDrop_OnLeave = frame:GetScript("OnLeave")
		frame:EnableMouse(true)
		frame:SetScript("OnLeave", GT_OnLeave)
	end
	frame:SetOwner(cell, "ANCHOR_NONE")
	frame:ClearAllPoints()
	local a1, a2 = GetAnchors(cell)
	frame:SetPoint(a1, cell, a2)	
end

local cell_tooltip_table = {}
local function OnUpdate()
	if cell_tooltip_table.frame then
		cell_tooltip_table.frame:SetPoint(cell_tooltip_table.a1, cell_tooltip_table.cell, cell_tooltip_table.a2)
	else
		local frame = EnumerateFrames()
		while frame do
			local _, relativeTo = frame:GetPoint()
			if relativeTo == cell then
				local a1, a2 = GetAnchors(cell)
				frame:SetPoint(a1, cell, a2)
				cell_tooltip_table.frame = frame
				cell_tooltip_table.cell = cell
				cell_tooltip_table.a1 = a1
				cell_tooltip_table.a2 = a2
				break
			end
			frame = EnumerateFrames(frame)
		end
	end
end

local function YssDrop_Click(cell, name, event, ...)
	local obj = broker:GetDataObjectByName(name)
	if obj.OnClick then
		obj.OnClick(cell, ...)
	end
end

local function YssDrop_Leave(cell, name, ...)
	YssDrop:SetScript('OnUpdate', nil)
	wipe(cell_tooltip_table)
	QTC.cellFunctions.OnLeave(cell)
	local obj = broker:GetDataObjectByName(name)
	if obj.OnLeave then
		obj.OnLeave(cell)
	end
	if MouseIsOver(GameTooltip) and (obj.tooltiptext or obj.OnTooltipShow) then return end

	if obj.tooltiptext or obj.OnTooltipShow then
		GT_OnLeave(GameTooltip)
	end
end

local function YssDrop_Enter(cell, name, ...)
	QTC.cellFunctions.OnEnter(cell)
	local obj = broker:GetDataObjectByName(name)
	if obj.tooltip then
		PrepareTooltip(obj.tooltip, cell)
		if obj.tooltiptext then
			obj.tooltip:SetText(obj.tooltiptext)
		end
		obj.tooltip:Show()
		
	elseif obj.OnTooltipShow then
		PrepareTooltip(GameTooltip, cell)
		obj.OnTooltipShow(GameTooltip)
		GameTooltip:Show()
	
	elseif obj.tooltiptext then
		PrepareTooltip(GameTooltip, cell)
		GameTooltip:SetText(obj.tooltiptext)
		GameTooltip:Show()		
	
	elseif obj.OnEnter then
		obj.OnEnter(cell)
		local frame = EnumerateFrames()
		while frame do
			local _, relativeTo = frame:GetPoint()
			if relativeTo == cell then
				local a1, a2 = GetAnchors(cell)
				frame:SetPoint(a1, cell, a2)
				YssDrop:SetScript('OnUpdate', OnUpdate)
				cell_tooltip_table.frame = frame
				cell_tooltip_table.cell = cell
				cell_tooltip_table.a1 = a1
				cell_tooltip_table.a2 = a2
				break
			end
			frame = EnumerateFrames(frame)
		end
		cell_tooltip_table.cell = cell
	end
end

function YssDrop:CreateBroker()
	self.obj = LibStub("LibDataBroker-1.1"):NewDataObject("YssDrop", {
		icon = "Interface\\AddOns\\YssDrop\\icon",
		label = "|cFF33FF99YssDrop|r",
		type = "data source",
		text  = "YssDrop",
	})
	self:SetupBroker()
end

function YssDrop:ShowTooltip(anchorFrame)
	if tooltip then self:HideTooltip() end
	tooltip = QTC:Acquire("YssDropTT", 1, "LEFT") -- or GameTooltip
	tooltip:SetFrameStrata(frameStrata[YssDrop.db.profile.Strata])
	
	tooltip:SetCallback("OnMouseDown", YssDrop_Click)
	tooltip:SetCallback("OnLeave", YssDrop_Leave)
	tooltip:SetCallback("OnEnter", YssDrop_Enter)
	tooltip:SetScale(YssDrop.db.profile.Scale)
	
	tooltip:AddNormalHeader("|cFF33FF99YssDrop|r")
	local count = 2
	local obj, state
	for i, name in ipairs(YssDrop.activebrokers) do
		obj = broker:GetDataObjectByName(name)
		if YssDrop.db.profile.Enabled[name] then
			tooltip:AddLine()
			if YssDrop.db.profile.Text[name] and YssDrop.db.profile.Label[name] then
				tooltip:SetCell(count, 1, string.format("|T%s:0|t %s: %s", obj.icon or '', obj.label or name, obj.text or ''), name)
			elseif YssDrop.db.profile.Text[name] then
				tooltip:SetCell(count, 1, string.format("|T%s:0|t %s", obj.icon or '', obj.text or ''), name)
			else
				tooltip:SetCell(count, 1, string.format("|T%s:0|t %s", obj.icon or '', obj.label or name), name)
			end
			count = count + 1
		end
	end
	if not self.db.profile.Click then
		tooltip:AddNormalLine('|cFF33FF99Click to config|r')
	end
	tooltip:SmartAnchorTo(anchorFrame)
	tooltip:EnableMouse()
	tooltip:Show()
end

function YssDrop:SetupBroker()
	if self.db.profile.Click then
		self.obj.OnEnter = nil
		self.obj.OnLeave = nil
		self.obj.OnClick = function(clickedFrame, button)
			if button == "RightButton" then 
				InterfaceOptionsFrame_OpenToCategory(YssDrop.optframe.YssDrop)
			else
				if tooltip then
					YssDrop:HideTooltip()
				else
					YssDrop:ShowTooltip(clickedFrame)
				end
			end
		end
		self.obj.OnTooltipShow = function(tt)
			tt:AddLine('YssDrop')
			tt:AddLine("|cffffff00Click|r to toggle YssDrop")
			tt:AddLine("|cffffff00Right-click|r to open the options menu")
		end
	else
		self.obj.OnTooltipShow = nil
		self.obj.OnClick = function(self, btn)
			InterfaceOptionsFrame_OpenToCategory(YssDrop.optframe.YssDrop)
		end
		self.obj.OnEnter = function(clickedFrame)
			YssDrop:ShowTooltip(clickedFrame)
		end
		self.obj.OnLeave = function ()
			if MouseIsOver( tooltip ) then
				tooltip:SetScript( 'OnLeave', function()
					if not MouseIsOver( tooltip ) then
						YssDrop:HideTooltip()
					end
				end)
			else
				YssDrop:HideTooltip()
			end
		end
	end
end

function YssDrop:Getter(info)
	return self.db.profile[info[#info]][info[#info-1]]
end

function YssDrop:Setter(info, val)
	self.db.profile[info[#info]][info[#info-1]] = val
end

 YssDrop.BrokerOptionTable = {
		type="group",
		dialogInline = true,
		name = function(info) return info[#info] end,
		order = function(info) return YssDrop.activebrokersRL[info[#info]]+100 end,
		args = {
			Enabled = {
				type = "toggle",
				name = 'Enable',
				order = 1,
				get = 'Getter',
				set = 'Setter',
				width = 'half',
			},
			Text = {
				type = "toggle",
				name = 'Text',
				order = 2,
				get = 'Getter',
				set = 'Setter',
				width = 'half',
			},
			Label = {
				type = "toggle",
				name = 'Label',
				order = 3,
				get = 'Getter',
				set = 'Setter',
				width = 'half',
			},
		}
	}

YssDrop.activebrokers = {}
YssDrop.activebrokersRL = {}
local slower = string.lower
local function SortComp(a,b)
	return slower(a) < slower(b)
end

function YssDrop:LibDataBroker_DataObjectCreated(event, name, obj)
	if name == 'YssDrop' or self.activebrokersRL[name] then return end -- do not alowe self to be displayed form self (causes all types of bad)
	self.activebrokers[#self.activebrokers+1] = name
	table.sort(self.activebrokers, SortComp)
	for i, name in ipairs(self.activebrokers) do
		if not self.MainOption.args[name] then
			self.MainOption.args[name] = self.BrokerOptionTable
		end
	end
	if not (self.db.profile.Enabled[name] ==  true or self.db.profile.Enabled[name] ==  false) then
		if obj.type == 'launcher' then
			self.db.profile.Enabled[name] = true
			self.db.profile.Text[name] = false
			self.db.profile.Label[name] = true
		else
			self.db.profile.Enabled[name] = false
			self.db.profile.Text[name] = true
			self.db.profile.Label[name] = true
		end
	end
	for i, name in ipairs(self.activebrokers) do
		self.activebrokersRL[name] = i
	end
end

function YssDrop:ADDON_LOADED(event, addon)
	if addon == "YssDrop" then
		
		self:UnregisterEvent("ADDON_LOADED")
		self.ADDON_LOADED = nil
		if IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent("PLAYER_LOGIN") end
	end
end

function YssDrop:PLAYER_LOGIN(event, addon)
	local defaults = {
		["profile"] = {
			['Enabled'] = {},
			['Text'] = {},
			['Label'] = {},
			['Scale'] = 1,
			['Strata'] = 7,
			['Click'] = false,
			['LibDBIcon'] = {
				['hide'] = true, 
			}
		},
	}
	self.db = LibStub("AceDB-3.0"):New("YssDrop_DB", defaults, "Default")
	self.db:RegisterCallback("OnProfileChanged", function() YssDrop:ReloadOptions() end)
	self.db:RegisterCallback("OnProfileCopied", function() YssDrop:ReloadOptions() end)
	self.db:RegisterCallback("OnProfileReset", function() YssDrop:ReloadOptions() end)
	self:RegisterModuleOptions('profile', LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db), 'Profile')
	self:CreateBroker()
	icon:Register("YssDrop", YssDrop.obj, YssDrop.db.profile.LibDBIcon)
	broker.RegisterCallback(YssDrop, "LibDataBroker_DataObjectCreated")
	self:ReloadOptions()
	self:UnregisterEvent("PLAYER_LOGIN")
	self.PLAYER_LOGIN = nil
end

function YssDrop:ReloadOptions()
	wipe(self.activebrokers)
	wipe(self.activebrokersRL)
	for name, obj in broker:DataObjectIterator() do
		self:LibDataBroker_DataObjectCreated(nil, name, obj)
	end
end

YssDrop.MainOption = {
	type="group",
	childGroups = 'select',
	handler = YssDrop,
	args = {
		scale = {
			name = 'Scale',
			order = 1,
			type = "range",
			min = .5,
			max = 2,
			step = .01,
			get = function(info)
				return YssDrop.db.profile.Scale
			end,
			set = function(info, val)
				YssDrop.db.profile.Scale = val
			end,
		},
		Strata = {
			type = "select",
			name = 'Tooltip Strata',
			order = 2,
			values = frameStrata,
			get = function() return  YssDrop.db.profile.Strata end,
			set = function(info, val)
				YssDrop.db.profile.Strata = val
			end,
		},
		Click = {
			type = "toggle",
			name = 'Dropdown on Click',
			desc = 'Show Dropdown on Click instead of on Hover.',
			order = 3,
			get = function() return  YssDrop.db.profile.Click end,
			set = function(info, val)
				YssDrop.db.profile.Click = val
				YssDrop:SetupBroker()
			end,
		},
		LibDBIcon = {
			type = "toggle",
			name = 'Minimap Icon',
			order = 4,
			get = function() return not YssDrop.db.profile.LibDBIcon.hide end,
			set = function(info, val)
				YssDrop.db.profile.LibDBIcon.hide = not val
				if val then
					icon:Show("YssDrop")
				else
					icon:Hide("YssDrop")
				end
			end,
		},
	},
}


YssDrop.optframe = {}
LibStub("AceConfig-3.0"):RegisterOptionsTable('YssDropConfig', YssDrop.MainOption)
YssDrop.optframe.YssDrop = LibStub("AceConfigDialog-3.0"):AddToBlizOptions('YssDropConfig', 'YssDrop')
-- function addapted from the RegisterModuleOptions function in Mapster
function YssDrop:RegisterModuleOptions(name, optionTbl, displayName)
	local cname = "YssDropConfig_"..name
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(cname, optionTbl)
	self.optframe[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(cname, displayName, "YssDrop")
end
